/* $Id: HelpViewer.java,v 1.2 2007/10/24 09:10:39 nito Exp $
 * 
 * Copyright (c)ARGO 21, Corporation. 2005, 2006.  All rights reserved.
 * 
 * This file is part of Nautica Workflow.
 * 
 *  Nautica Workflow is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Nautica Workflow is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Nautica Workflow; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 */
package jp.co.argo21.nautica.commons.swing;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.IOException;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

/**
 * ヘルプビューアを実現する。ヘルプファイルはHTML文書として記述する。
 * ヘルプ内容を記載したHTMLファイルをjarにまとめておき、
 * CLASSPATHに通すことで容易にアクセス可能になる。
 *
 * @author  Norihiro Itoh(ARGO 21 Corp.)
 * @version $Revision: 1.2 $
 * @since   Nautica Workflow 1.0
 */
public class HelpViewer extends JFrame
{
	/** シリアルバージョンID */
	private static final long serialVersionUID = 1L;

	private static final String BUNDLE_PATH
		= "jp.co.argo21.nautica.commons.swing.resources.commons-swing";
	private static String PAGE_NOT_FOUND;
	private static ResourceBundle resources;

	private BrowserPane browserPane;

	static
	{
		try {
			resources = ResourceBundle.getBundle(BUNDLE_PATH);
			PAGE_NOT_FOUND = resources.getString("HelpViewer.nopage");
		} catch (MissingResourceException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * ヘルプビューアを構築する。
	 *
	 * @param path リソースバンドルのパス名
	 */
	public HelpViewer()
	{
		super(resources.getString("HelpViewer.title"));//最小化可

		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int sw = screen.width;
		int sh = screen.height;
		Dimension size = new Dimension(500, (int)(sh * 0.90));
		this.setSize(size);
		this.setLocation(sw - size.width, 0);

		browserPane = new BrowserPane();
		this.setContentPane(browserPane);
		this.validate();
	}

	/**
	 * 指定されたリソースをブラウザにロードする。
	 *
	 * @param s リソースパス
	 */
	public void loadResource(String path)
	{
		URL url = null;
		if (path != null) url = HelpViewer.class.getResource(path);
		if (url == null) url = HelpViewer.class.getResource(PAGE_NOT_FOUND);
		browserPane.loadURL(url);
	}

	/**
	 * 指定されたURLをブラウザにロードする。
	 *
	 * @param urlpath URL文字列
	 */
	public void loadURL(String urlpath)
	{
		URL url = null;
		try {
			url = new URL(urlpath); 
		} catch (Exception ex) {
			url = HelpViewer.class.getResource(PAGE_NOT_FOUND);
		}
		browserPane.loadURL(url);
	}

	/**
	 * 指定されたURLをブラウザにロードする。
	 *
	 * @param url URL
	 */
	public void loadURL(URL url)
	{
		if (url == null) url = HelpViewer.class.getResource(PAGE_NOT_FOUND);
		browserPane.loadURL(url);
	}

	/**
	 * ヘルプブラウザのブラウザ部分を実現するクラスである。
	 *
	 * @author  ARGO21 Corp.
	 * @version $Revision: 1.2 $
	 * @since   Nautica Workflow 1.0
	 */
	private final class BrowserPane extends JScrollPane
	implements HyperlinkListener
	{
		/** シリアルバージョンID */
		private static final long serialVersionUID = 1L;

		/**
		 * ブラウザ表示部
		 */
		private JEditorPane browser;

		/**
		 * ブラウザのインスタンスを生成する。
		 */
		private BrowserPane()
		{
			browser = new JEditorPane();
			browser.setEditable(false);
			browser.addHyperlinkListener(this);

			JViewport vp = getViewport();
			vp.add(browser);
		}

		/**
		 * ハイパーテキストリンクが更新されたときに呼び出される。
		 *
		 * @param e 更新を処理するイベント
		 * @see javax.swing.event.HyperlinkListener
		 */
		public void hyperlinkUpdate(HyperlinkEvent e)
		{
			if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
				linkActivated(e.getURL());
			}
		}

		/**
		 * 指定されたURLをブラウザにロードする。
		 *
		 * @param u URL
		 */
		public void loadURL(URL u)
		{
			SwingUtilities.invokeLater(new PageLoader(u));
		}

		/**
		 * リンクが有効になったことを通知する。
		 *
		 * @param u URL
		 */
		private void linkActivated(URL u)
		{
			SwingUtilities.invokeLater(new PageLoader(u));
		}

		/**
		 * ページをロードするスレッドを実現するクラスである。
		 *
		 */
		private final class PageLoader implements Runnable
		{
			/**
			 * URL
			 */
			private URL url;

			/**
			 * ページローダのインスタンスを生成する。
			 *
			 * @param u URL
			 */
			PageLoader(URL u) {
				url = u;
			}

			/**
			 * ページをロードする。
			 */
			public void run() {
				if (url == null) {
					Container parent = browser.getParent();
					parent.repaint();
				} else {
					javax.swing.text.Document doc = browser.getDocument();
					try {
						browser.setPage(url);
					} catch (IOException ioe) {
						browser.setDocument(doc);
					} finally {
						url = null;
						SwingUtilities.invokeLater(this);
					}
				}
			}
		}
	}
}
