/* $Id: NumericDocument.java,v 1.2 2007/10/24 09:10:39 nito Exp $
 * 
 * Copyright (c)ARGO 21, Corporation. 2005, 2006.  All rights reserved.
 * 
 * This file is part of Nautica Workflow.
 * 
 *  Nautica Workflow is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Nautica Workflow is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Nautica Workflow; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 */
package jp.co.argo21.nautica.commons.swing;

import javax.swing.text.*;

/**
 * 数値入力用フィールドのための制御を行う。
 *
 * @author  Norihiro Itoh(ARGO 21 Corp.)
 * @version $Revision: 1.2 $
 * @since   Nautica Workflow 1.0
 */
public class NumericDocument extends ByteLimitedDocument
{
	/** シリアルバージョンID */
	private static final long serialVersionUID = 1L;

	/** 小数点が含まれているかどうかのフラグ */
	private boolean hasPoint   = false;
	/** マイナス符号が含まれているかどうかのフラグ */
	private boolean hasSign    = false;
	/** 入力を正の数に限定するかどうかのフラグ。trueの場合、0以上の入力のみ許す。 */
	private boolean isPositive = false;
	/** 入力を整数に限定するかどうかのフラグ。trueの場合、整数の入力のみ許す。 */
	private boolean isInteger  = false;

	/**
	 * 数値入力制限用ドキュメントオブジェクトを生成する。
	 *
	 * @param limit 入力制限するバイト数
	 */
	public NumericDocument(int limit)
	{
		super(limit);
	}

	/**
	 * 数値入力制限用ドキュメントオブジェクトを生成する。
	 *
	 * @param limit     入力制限するバイト数
	 * @param isInteger 入力を整数に限定するかを表すフラグ
	 */
	public NumericDocument(int limit, boolean isInteger)
	{
		super(limit);
		this.isInteger = isInteger;
	}

	/**
	 * 数値入力制限用ドキュメントオブジェクトを生成する。
	 *
	 * @param limit      入力制限するバイト数
	 * @param isInteger  入力を整数に限定するかを表すフラグ
	 * @param isPositive 入力を正の数に限定するかを表すフラグ
	 */
	public NumericDocument(int limit, boolean isInteger, boolean isPositive)
	{
		super(limit);
		this.isInteger = isInteger;
		this.isPositive = isPositive;
	}

	/**
	 * 文字列挿入時の制御を行う。
	 *
	 * @param offset 挿入位置
	 * @param s      挿入文字列
	 * @param a      属性値
	 * @throws BadLocationException 指定された挿入の位置が、ドキュメント内の有効な位置ではない場合
	 */
	public void insertString(int offset, String s, AttributeSet a) throws BadLocationException
	{
		if (s == null) return;

		char[] chars = s.toCharArray();
		int max = chars.length;
		boolean tempPoint = hasPoint;
		boolean tempSign = hasSign;

		for (int i = 0; i < max; i++) {
			char c = chars[i];
			switch (c) {
			case '.':
				if (isInteger) return;
				if (tempPoint) return;
				if (offset == 0) return;
				if (i != 0) return;
				char c2 = getText(0, 1).charAt(0);
				if (c2 == '-') {
					if (offset < 2) return;
				}
				tempPoint = true;
				break;
			case '-':
				if (isPositive) return;
				if (tempSign) return;
				if (offset != 0) return;
				tempSign = true;
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				break;
			default:
				return;
			}
		}

		super.insertString(offset, s, a);

		hasPoint = tempPoint;
		hasSign = tempSign;
	}

	/**
	 * 文字列削除時の制御を行う。
	 *
	 * @param offset 削除位置
	 * @param len    削除長
	 * @throws BadLocationException 指定された削除の位置が、ドキュメント内の有効な位置ではない場合
	 */
	public void remove(int offset, int len) throws BadLocationException
	{
		boolean tempPoint = hasPoint;
		boolean tempSign = hasSign;

		String s = getText(offset, len);

		if (s.indexOf('.') > -1) tempPoint = false;
		if (s.indexOf('-') > -1) tempSign = false;

		super.remove(offset, len);

		hasPoint = tempPoint;
		hasSign = tempSign;
	}
}
